# -*- coding: utf-8 -*- 
# @Time    : 04/03/2024 
# @Author  : nzooherd 
# @File    : api.py 
# @Software: Visual Studio Code


import functools
import inspect
from logging import getLogger
from typing import Any, Callable, Coroutine, Self, Set, Tuple
from venv import logger

from telethon.events import filters

from aspect.message import MessageAspect
from aspect.user import UserAspect

class ApiAspect:

    logger = getLogger()

    HandlerApis: Set[Tuple[Self, Any, Callable]] = set()

    def __init__(self, filter: filters.Filter = None, message_aspect=None, user_aspect=None, **kwargs) -> None:
        self.filter = filter
        self.aspects = [MessageAspect() if not message_aspect else message_aspect,
                        UserAspect() if not user_aspect else user_aspect]
        
        self.kwargs = kwargs

    def __call__(self, handler: Callable[[Any, Any], Coroutine]):
        wrapper = handler
        for aspect in self.aspects:
            wrapper = aspect(wrapper)
        
        annotations = inspect.get_annotations(handler)
        event = list(annotations.values())[0] if len(annotations) > 0 else None
        @functools.wraps(wrapper)
        async def api_wrapper(controller: object, update: Any):
            try:
                await wrapper(controller, update)
            except Exception as e:
                logger.exception(e)

        ApiAspect.HandlerApis.add((self, event, api_wrapper))
        return api_wrapper